/*****************************************************************************
*
* Copyright (C) 2014      Advanced Digital Chips, Inc. All Rights Reserved.
*						http://www.adc.co.kr
*
* THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
* EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE
*
*****************************************************************************/

#include "sdk.h"
#include "etc_driver/AK4183.h"
void lcdon();
void port_init();
void RSP_Run();
void mass_storage_main();
void display_clock_init();
void run_app();

void lcd_bright(U32 percent);
//#define EGL_MODE

#ifdef EGL_MODE
BOOL my_touchinput( EGL_MSG* pMsg )
{
    BOOL touchdown;
    EGL_POINT touch_pt;
    if( process_touch( &touchdown, &touch_pt ) )
    {
        if( touchdown )
            pMsg->msgID = EGL_MSG_TOUCHED;
        else
            pMsg->msgID = EGL_MSG_UNTOUCHED;
        pMsg->param.point.x = touch_pt.x;
        pMsg->param.point.y = touch_pt.y;
        return TRUE;
    }
    return FALSE;
}
BOOL process_touch( BOOL* touchdown, EGL_POINT* pPoint )
{
    static BOOL mypushed = FALSE;
    static int oldx, oldy;
    if( get_touchxy( &pPoint->x, &pPoint->y ) )
    {
        mypushed = TRUE;
        oldx = pPoint->x;
        oldy = pPoint->y;
        *touchdown = TRUE;
        return TRUE;
    }
    else
    {
        if( mypushed )
        {
            mypushed = FALSE;
            pPoint->x = oldx ;
            pPoint->y = oldy ;
            *touchdown = FALSE;
            return TRUE;
        }
        return FALSE;
    }
    return FALSE;
}



EGL_HANDLE hWin;
EGL_HANDLE hUSB;
EGL_HANDLE hCon;
EGL_HANDLE hRun;
void usbbtn_cb( EGL_HANDLE h, int evt )
{
    if( evt == BTN_CLICKED )
    {
        PRINTLINE;
        egl_show_object( hCon, FALSE );
        egl_show_object( hRun, FALSE );
        egl_draw();
        mass_storage_main();//never return
    }
}
void conbtn_cb( EGL_HANDLE h, int evt )
{
    if( evt == BTN_CLICKED )
    {
        PRINTLINE;
        egl_show_object( hUSB, FALSE );
        egl_show_object( hRun, FALSE );
        egl_draw();
        RSP_Run();//never return
    }
}
void runbtn_cb( EGL_HANDLE h, int evt )
{
    if( evt == BTN_CLICKED )
    {
        char* fname = "boot.bin";
        U32* buf = ( U32* )0xc0000000;
        FIL fp;
        int size;
        U32 nRead;
        FRESULT res = f_open( &fp, fname, FA_READ | FA_OPEN_EXISTING );
        if( res == FR_OK )
        {
            size = f_size( &fp );
            f_read( &fp, buf, size, &nRead );
            f_close( &fp );
            void ( *startfp )() = ( void ( * )() )buf[0]; // get start address
            DEBUGPRINTF( "Start Addres : 0x%x\r\n", startfp );
            startfp();
        }
    }
}
#else
bool check_collision( int startx, int starty, int endx, int endy, int x, int y )
{
    if( ( startx < x ) &&  ( endx > x ) )
    {
        if( ( starty < y ) &&  ( endy > y ) )
        {
            return true;
        }
    }
    return false;
}
bool check_collisionrect( RECT* pRect, int x, int y )
{
    int endx = pRect->x + pRect->w;
    int endy = pRect->y + pRect->h;
    int startx = pRect->x;
    int starty = pRect->y;
    return check_collision( startx, starty, endx, endy, x, y );
}
#endif
extern unsigned char connect_jpg[];
extern unsigned int connect_jpg_len;
extern unsigned char USB_jpg[];
extern unsigned int USB_jpg_len;
extern unsigned char Run_jpg[];
extern unsigned int Run_jpg_len;
int main()
{
    port_init();
    uart_config( 0, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE );
    debugstring( "================================================\r\n" );
    debugprintf( "AMAZON2 bootloader Program.System Clock(%dMhz)\r\n", get_axi_clock() / 1000000 );
    debugstring( "================================================\r\n" );
    FATFS fs_nand;
    f_mount( DRIVE_NAND, &fs_nand );
    f_chdrive( DRIVE_NAND );
	display_clock_init();
	/*
	if LCD's resolution is 800x480.
	Frame-buffer size = 1024 * 512 * 2 (RGB565) * 2(doube buffer) = 2Mbyte
	Frame-buffer size = 1024 * 512 * 4 (RGB888) * 2(doube buffer) = 4Mbyte

	if LCD's resolution is 800x600.
	Frame-buffer size = 1024 * 1024 * 2 (RGB565) * 2(doube buffer) = 4Mbyte
	Frame-buffer size = 1024 * 1024 * 4 (RGB888) * 2(doube buffer) = 8Mbyte

	texture memory start address = frame buffer start + frame buffer size
	*/
	ge_set_texture_mem(0xc2900000, 0x5700000);
	setscreenex(0xc2100000, 800, 480, SCREENMODE_RGB565 | SCREENMODE_DOUBLEBUF, 928, (40 << 16) | 88, (128 << 16) | 928, 525, (13 << 16) | 16, (45 << 16) | 525, VMODE_RGB);

    draw_rectfill( 0, 0, 1024, 600, MAKE_COLORREF( 255, 255, 255 ) );
    flip();
    lcdon();
    SURFACE* surf_run;
    SURFACE* surf_usbcon;
    SURFACE* surf_usbmass;
    surf_usbcon = loadjpgp_hw( ( U32 )connect_jpg, connect_jpg_len );
    surf_usbmass = loadjpgp_hw( ( U32 )USB_jpg, USB_jpg_len );
    surf_run = loadjpgp_hw( ( U32 )Run_jpg, Run_jpg_len );
    twi_set_freq( 100000 );
    touchinit();
#ifdef EGL_MODE
    egl_init();
    hWin = egl_create_window( "MAIN" );
    hUSB = egl_create_image_button( surf_usbmass, surf_usbmass, 150, 300, 96, 96 );
    egl_image_button_callback( hUSB, usbbtn_cb );
    egl_window_add_object( hWin, hUSB );
    hCon = egl_create_image_button( surf_usbcon, surf_usbcon, 500, 300, 96, 96 );
    egl_image_button_callback( hCon, conbtn_cb );
    egl_window_add_object( hWin, hCon );
    hRun = egl_create_image_button( surf_run, surf_run, 800, 300, 96, 96 );
    egl_image_button_callback( hRun, runbtn_cb );
    egl_window_add_object( hWin, hRun );
    egl_window_show( hWin, TRUE );
    while( 1 )
    {
        EGL_POINT touch_pt;
        BOOL touchdown = FALSE;
        if( process_touch( &touchdown, &touch_pt ) )
            egl_user_touch_input( touchdown, &touch_pt );
        egl_draw();
    }
#else
    EGL_FONT* font = create_bitfont();
    draw_rectfill( 0, 0, get_screen_width(), get_screen_height(), MAKE_COLORREF( 255, 255, 255 ) );
    RECT rect_con = {150, 200, 96, 96};
    RECT rect_usb = {350, 200, 96, 96};
    RECT rect_run = {550, 200, 96, 96};
    draw_surface( surf_usbcon, rect_con.x, rect_con.y );
    draw_text( font, 130, 300, "USB Communication" );
    draw_surface( surf_usbmass, rect_usb.x, rect_usb.y );
    draw_text( font, 330, 300, "USB Mass Storage" );
    draw_surface( surf_run, rect_run.x, rect_run.y );
    draw_text( font, 530, 300, "Run \"boot.bin\" at NAND" );
    flip();
    while( 1 )
    {
        int x, y;
        if( get_touchxy( &x, &y ) )
        {
            if( check_collisionrect( &rect_usb, x, y ) == true )
            {
                draw_rectfill( 0, 0, get_screen_width(), get_screen_height(), MAKE_COLORREF( 255, 0, 0 ) );
                draw_surface( surf_usbmass, rect_usb.x, rect_usb.y );
                flip();
                mass_storage_main();
            }
            else if( check_collisionrect( &rect_con, x, y ) == true )
            {
                draw_rectfill( 0, 0, get_screen_width(), get_screen_height(), MAKE_COLORREF( 0, 255, 0 ) );
                draw_surface( surf_usbcon, rect_con.x, rect_con.y );
                flip();
                lcd_bright( 0 );
                *( volatile U32* )0x80000008 = 0; // graphic engine off.
                RSP_Run();
            }
            else if( check_collisionrect( &rect_run, x, y ) == true )
            {
                draw_rectfill( 0, 0, get_screen_width(), get_screen_height(), MAKE_COLORREF( 0, 0, 255 ) );
                draw_surface( surf_run, rect_run.x, rect_run.y );
                flip();
                FATFS fs_nand;
                f_mount( DRIVE_NAND, &fs_nand );
                f_chdrive( DRIVE_NAND );
                char* fname = "boot.bin";
                U32* buf = ( U32* )0xc0000000;
                FIL fp;
                int size;
                U32 nRead;
                FRESULT res = f_open( &fp, fname, FA_READ | FA_OPEN_EXISTING );
                if( res == FR_OK )
                {
                    size = f_size( &fp );
                    f_read( &fp, buf, size, &nRead );
                    f_close( &fp );
                    void ( *startfp )() = ( void ( * )() )buf[0]; // get start address
                    lcd_bright( 0 );
                    *( volatile U32* )0x80000008 = 0; // graphic engine off.
                    startfp();
                }
                DEBUGPRINTF( "Cannot open : %s\r\n", fname );
                draw_surface( surf_usbcon, rect_con.x, rect_con.y );
                draw_surface( surf_usbmass, rect_usb.x, rect_usb.y );
                draw_surface( surf_run, rect_run.x, rect_run.y );
                flip();
            }
            else
            {
                DEBUGPRINTF( "invalid area\r\n" );
            }
        }
    }
#endif
    while( 1 );
    return 0;
}
